/*
//FILENAME:Bank.java
//CLASSES:Bank
//AUTHOR:Donovan Walker
//CLASS:CSE445 FALL 2K Graduate project
*/

//import classes
import BankAccount;
import BankFrame;
import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import Transaction;

//import interfaces
import BankTransactionInterface;
import TransactionInterface;


class Bank implements  BankTransactionInterface, AccountListInterface
{
	
	//CONSTRUCTORS
	
	Bank(String inBankId)
	{
		//Initialize class variables
		sBankId = inBankId;
		bConnected = false;
		
		//Load Customer accounts
		vAccounts = loadAccounts();
		
	}
	
	
	Bank(String inBankId, String inTransactionManager)  //DONE
	{
		//Initialize class variables
		sBankId = inBankId;
		bConnected = false;
		
		//Load Customer accounts
		vAccounts = loadAccounts();
		
		//Connect to TransactionManager
		connect(inTransactionManager);
		
	}
	
	
	//METHODS
	
	private boolean accountIsOurs(String inAccNumString) //DONE
	{
		//check to see if first sBankId.length() chars of Account number are the same as the banks ID.
		//if true account should be able to be found @ this bank
		if(inAccNumString.startsWith(sBankId))
		{
			return (true);
		}
		else
		{
			return (false);
		}

	}
	
	
	public void closeBank()
	{
		if(tiServer != null)
		{
			try
			{

				tiServer.remove(sBankId);

			}
			catch(Exception e)
			{
				System.out.println(e.getMessage());
			}
			finally
			{
				saveAccounts();
			}
		}
	}
	
	
	public boolean connect(String inTMAddress)  //DONE
	{
		//connect to the transaction server for access to accounts not belonging to this bank
		try
		{
			
			tiServer = (TransactionInterface)Naming.lookup("rmi://" + inTMAddress + "/TransactionInterface");
			UnicastRemoteObject.exportObject(this);  //Export to Transaction Manager
			
			//Join bank list on Transaction Manager.  Print result
			if(tiServer.join(this) == true) 
			{
				System.out.println("Connected");
				bConnected = true;
				return (true);
			}
			
			else
			{
				System.out.println("Not connected.  Returned false");
				bConnected = false;
				return(false);
			}
		}
		
		catch(Exception e)
		{
			System.out.println("Exception: " + e.getMessage());
			return(false);
		}
			
	}

	
	public void display()
	{
		BankAccount baAccount;
		
		System.out.println("\n" + vAccounts.size() + " accounts in bank: " + sBankId);
		for(int i = 0; i < vAccounts.size(); i++)
		{
			baAccount = (BankAccount)vAccounts.get(i); 
			System.out.println("Account " + i + "Name:" + baAccount.getName() + " Num:" + baAccount.getNum() + " Bal:" + baAccount.getBalance());
		}
	}


	public String getBalace(String inAccount, String inPassword) //DONE
	{
		
		BankAccount tempAccount;
		boolean found						= false;
		int i										= 0;
		
		//if account is found at this bank...
		if(accountIsOurs(inAccount))
		{
			//find it in the vector
			while((found == false) && (i < vAccounts.size()))
			{
				tempAccount = (BankAccount)vAccounts.get(i);
				
				if( (tempAccount.getNum()).equals(inAccount) )  //check for account
				{
					return(TRANS_SUCCED + REM_BAL + tempAccount.getBalance());
				}
				i++;
			}
			
			//can't find account
			return(TRANS_FAILED + NOT_FOUND);
		}
		
		else
	 	{
			try
			{
				//Call remote object to conduct transaction 
				return ( tiServer.getRemoteBalance( new Transaction(inAccount, inPassword, TYPE_CHKBAL, 0) ) );
			}
			catch(Exception e)
			{
				System.out.print("Error :" + e);
			}
		}
		
		return (null);
		
	}
	

	public String getCash(String inAccount, String inPassword, float inAmount) //DONE
	{
		
		BankAccount tempAccount;
		int     i     						= 0;
		float   remainder					= 0;
		Transaction tThisTransaction;
		
		if(accountIsOurs(inAccount))
		{
			while(i < vAccounts.size())
			{
				tempAccount = (BankAccount)vAccounts.get(i);
				
				if( (tempAccount.getNum()).equals(inAccount) )
				{
					remainder = tempAccount.getBalance() - inAmount;
					if(remainder >= 0)
					{
						tempAccount.setBalance(remainder);
						saveAccounts();
						return(TRANS_SUCCED + REM_BAL + remainder);
					}
					
					else
					{
						return(TRANS_FAILED + SHORT_FUNDS);
					}
				}
				i++;
			}
			
			return(TRANS_FAILED + NOT_FOUND);
		}

		else
		{
			//test code
			System.out.println("Querying server");
			try
			{
				return ( tiServer.getRemoteCash( new Transaction(inAccount, inPassword, TYPE_WTHDRW, inAmount) ) );
			}
			catch(Exception e)
			{
				System.out.print("Error :" + e);
			}
			
		}
		
		return(null);
	
	}
	
	
	//Load accounts from file into vector and return
	private Vector loadAccounts() //DONE
	{
		Vector tempV = new Vector(10, 10); //
		
		try
		{
			ObjectInputStream in = new ObjectInputStream(new FileInputStream(sBankId + ACCOUNT_FILE));
			tempV = (Vector)in.readObject();
			in.close();
			return (tempV);
		}
		catch(Exception e)
		{
			System.out.print("Error: " + e);
			return (tempV);
		}
	}


	public static void main(String args[])
	{
		//variables
		Bank thisBank;
		BankFrame thisFrame;
		boolean   connected = false;
		
		//check to see if bank specified on the command line.  If ! then display appropriate UI
		if(args.length == 1)
		{
			
			thisBank 	= new Bank(args[0]);
			thisFrame = new BankFrame(("Bank " + args[0]), thisBank);
			thisFrame.show();
		}
		
		else if(args.length > 1)
		{
			thisBank 	= new Bank(args[0], args[1]);
			thisFrame = new BankFrame(("Bank " + args[0]), thisBank);
			thisFrame.show();
		}
		
		else
		{
			printInstructions();
		}


	}
	
	//Makes a new account and adds this Banks Id num to beginning of inAccNum
	public void newAccount(String inName, String inPassword, String inAccNum, float inBalance) //DONE
	{
		BankAccount baAccount = new BankAccount(inName, inPassword, (sBankId + inAccNum), inBalance);
		vAccounts.add(baAccount);
		saveAccounts();
	}
	
	
	public static void printInstructions()
	{
		System.out.println("You need to enter one or more command line arguments");
		System.out.println("arg[0] = bankname, arg[1] = ip of bankswitch");
		System.out.println("bankname: the this will also specify beginning of account file name");
		System.out.println("ip of...: the ip of the switch server for connecting to other banks");
		System.out.println("arg[1] may be omitted");
	}
	
	
	public boolean saveAccounts() //DONE
	{
		try
		{
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(sBankId + ACCOUNT_FILE));
			out.writeObject(vAccounts);
			out.close();
			return (true);
		}
		catch(Exception e)
		{
			System.out.print("Error: " + e);
			return (false);
		}
	}
	
	
	
	public boolean saveAccounts(String inPath) //DONE
	{
		try
		{
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(inPath));
			out.writeObject(vAccounts);
			out.close();
			return (true);
		}
		catch(Exception e)
		{
			System.out.print("Error: " + e);
			return (false);
		}
	}
	
	
	
	
	//INTERFACE  /AccountListInterface
	
	//this interface implemented to protect the Vector vAccounts from tampering by other classes
	public String[] getAccountList()
	{
		BankAccount baTemp = null;
		String[] sTemp = null;
		
		sTemp = new String[vAccounts.size()];
		
		for(int i = 0; i < vAccounts.size(); i++)
		{
			baTemp = (BankAccount)vAccounts.get(i);

			sTemp[i] = new String(ACCOUNT_NUM + baTemp.getNum() + " " + ACCOUNT_NAME + baTemp.getName() + " " + ACCOUNT_BAL + baTemp.getBalance());
				
		}
		return(sTemp);
	}
	
	//INTERFACE  /BankTransactionInterface
	
	public String getBankId() throws java.rmi.RemoteException //DONE
	{
		return (sBankId);
	}
	
	
	public String getRemoteBalance(Transaction inTransaction) throws java.rmi.RemoteException  //DONE
	{
		
		BankAccount tempAccount;
		int     i     						= 0;
		float   remainder					= 0;
		Transaction tThisTransaction;
		
		if(accountIsOurs(inTransaction.sAccountNum))
		{
			while(i < vAccounts.size())
			{
				tempAccount = (BankAccount)vAccounts.get(i);
				
				if( (tempAccount.getNum()).equals(inTransaction.sAccountNum) )
				{
					return(TRANS_SUCCED + REM_BAL + tempAccount.getBalance());
				}
				i++;
			}

		}
		
		return(TRANS_FAILED + NOT_FOUND);

	}
		
	
	public String getRemoteCash(Transaction inTransaction) throws java.rmi.RemoteException //DONE
	{
		
		BankAccount tempAccount;
		int     i     						= 0;
		float   remainder					= 0;
		Transaction tThisTransaction;
		
		if(accountIsOurs(inTransaction.sAccountNum))
		{
			while(i < vAccounts.size())
			{
				tempAccount = (BankAccount)vAccounts.get(i);
				
				if( (tempAccount.getNum()).equals(inTransaction.sAccountNum) )
				{
					remainder = tempAccount.getBalance() - inTransaction.fAmount;
					if(remainder >= 0)
					{
						tempAccount.setBalance(remainder);
						saveAccounts();
						return(TRANS_SUCCED + REM_BAL + remainder);
					}
					
					else
					{
						return(TRANS_FAILED + SHORT_FUNDS);
					}
				}
				i++;
			}

		}
		
		return(TRANS_FAILED + NOT_FOUND);
	}
	
	
	//VARIABLES

	protected boolean								bConnected;
	protected String 								sBankId;
	private		TransactionInterface 	tiServer;
	private		Vector								vAccounts;
	
	//CONSTANTS
	static final String ACCOUNT_FILE = "accounts.bdf";
	static final String ACCOUNT_BAL  = "AccBal: ";
	static final String ACCOUNT_NUM  = "AccNum: ";
	static final String ACCOUNT_NAME = "AccNam: ";
	static final String TRANS_FAILED = "Transaction failed: ";
	static final String TRANS_SUCCED = "Transaction success: ";
	static final String NOT_FOUND    = "Account not found ";
	static final String SHORT_FUNDS  = "Insufficient funds ";
	static final String PASSWD       = "Invalid password ";
	static final String NO_ACCOUNT   = "Invalid account #";
	static final String REM_BAL   	 = "Remaining Balance $";
	static final String TYPE_WTHDRW  = "Withdraw";
	static final String TYPE_CHKBAL  = "Balance";
 
}

