/*
//FILENAME:BankFrame.java
//CLASSES: BankFrame
//AUTHOR:Donovan Walker
//CLASS:CSE445 Fall 2K Grad project
*/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

class BankFrame extends JFrame implements ActionListener
{
	
	//CONSTRUCTORS
	
	BankFrame(String inTitle, Bank inBank)
	{
		
		sTitle = inTitle;
		setTitle(sTitle);
		
		bBank = inBank;
    
		addWindowListener(new WindowAdapter()
    {  
			public void windowClosing(WindowEvent e)
      {
				bBank.closeBank();
				System.exit(0);
      }
    } 
		);
		
		//Setup the Frame on the screen
    Toolkit tk = Toolkit.getDefaultToolkit();
    Dimension d = tk.getScreenSize();
    int screenHeight = d.height;
    int screenWidth = d.width;
    setSize((int)(screenWidth / 2.5), (int)(screenHeight / 2.5));
    setLocation((int)(screenWidth / 3.4), (int)(screenHeight / 3.4));
		
		
		//Prep for setting up GUI components
		
		//Get the content pane
		cp = getContentPane();
		
		pATM 		 = new ATMPanel(this);
		pAccount = new AccountPanel(this);
		fAccountList = new AccountsFrame(bBank, bBank.sBankId);
		pConnect = new ConnectPanel(this);
		
		if(inBank.bConnected)//if bank is connected show the ATM window
		{
			//Set the title if connected
			setTitle(sTitle + " " + ACCOUNT + " Connected!");
			//Setup the MenuBar
			bMenuBar = new BankMenuBar(this);
			cp.add(pATM);
			setJMenuBar(bMenuBar);
		
		}
		
		else
		{
			//Set the title if not connected
			setTitle(sTitle + " " + CONNECT + " Not Connected");
			//if bank is not connected show the connect panel
			bMenuBar = new BankMenuBar(this);
			cp.add(pConnect);
			setJMenuBar(bMenuBar);
		
		}

	}
	
	
	//returns the connection status of the bank
	public boolean isConnected()
	{
		return(bBank.bConnected);
	}
	
	
	//INTERFACE  /ACTIONLISTENER
	
	public void actionPerformed(ActionEvent evt)
	{
		//variables
		float	 fInitBal  = 0;
		float  fWdAmount = 0;
		String arg = null;
		
		arg = evt.getActionCommand();
		
		//show the ATM panel
		if(arg.equals("ATM Panel"))
		{
			setTitle(sTitle + " " + ATM);
			pATM.setVisible(true);
			pAccount.setVisible(false);
			pConnect.setVisible(false);
			cp.add(pATM);
			cp.remove(pAccount);
			cp.remove(pConnect);
			
		}
		
		//show the Accounts panel
		else if(arg.equals("Accounts Panel"))
		{
	
			setTitle(sTitle + " " + ACCOUNT);
			pATM.setVisible(false);
			pAccount.setVisible(true);
			pConnect.setVisible(false);
			cp.add(pAccount);
			cp.remove(pATM);
			cp.remove(pConnect);
			fAccountList.show();
		
		}
		
		//show the Connect panel
		else if(arg.equals("Connect Panel"))
		{

			setTitle(sTitle + " " + CONNECT);
			pATM.setVisible(false);
			pAccount.setVisible(false);
			pConnect.setVisible(true);
			cp.remove(pAccount);
			cp.remove(pATM);
			cp.add(pConnect);
		
		}
		
		//add an account
		else if(arg.equals("Add"))
		{
			try
			{
				fInitBal = Float.parseFloat(pAccount.jtInitBal.getText());
				bBank.newAccount(pAccount.jtAccountName.getText(),(null), pAccount.jtAccountNum.getText(), fInitBal);
			}
			
			catch(Exception e)
			{
					pAccount.jlStatusLabel.setText(pAccount.STATUS_HEADER + "You must enter a valid withdraw amount");
			}
			
		}
		
		//attempt to connect
		else if(arg.equals("Connect"))
		{
			if(bBank.connect(pConnect.jtIpAddress.getText()) )
			{
				pConnect.jlStatusLabel.setText(pConnect.STATUS_HEADER + pConnect.CONNECTED);
				setTitle(sTitle + " " + ACCOUNT);
			}
		}
		
		//Withdraw money from account
		else if(arg.equals("Submit"))
		{
			try
			{
				
				fWdAmount = Float.parseFloat(pATM.jtWdAmount.getText());
				pATM.jlStatusLabel.setText(pATM.STATUSHEADER + bBank.getCash(pATM.jtAccountNum.getText(), (null), fWdAmount));
			}
			
			catch(Exception e)
			{
					pATM.jlStatusLabel.setText(pATM.STATUSHEADER + "You must enter a valid withdraw amount");
			}
			
		}
		
		else if(arg.equals("Show Accounts"))
		{
			fAccountList.show();
		}
				
		//Get outa this program
		else if(arg.equals("Exit"))
		{
			
			bBank.closeBank();
			System.exit(0);
		}
	}
	
	
	//VARIABLES
	
	private AccountPanel 	pAccount;
	private AccountsFrame fAccountList;
	private ATMPanel 			pATM;
	private Bank					bBank;
	private BankMenuBar 	bMenuBar;
	private ConnectPanel 	pConnect;
	private Container 		cp;		//for conentpane
	private String				sTitle;
	
	
	//CONSTANTS
	private static String ACCOUNT = "Add Accounts";
	private static String ATM			= "ATM Machine";
	private static String CONNECT = "Connect Window";

}
