/*
//FILENAME:BankSwitch.java
//CLASSES:BankSwitch
//AUTHOR:Donovan Walker
//CLASS:CSE445 FALL 2K Grad Proj
*/

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.text.*;
import java.util.*;

import BankTransactionInterface;

class BankSwitch extends UnicastRemoteObject implements TransactionInterface
{
	
	//CONSTRUCTORS
	
	BankSwitch() throws java.rmi.RemoteException
	{
		bsfOutFrame = new BankSwitchFrame(this);
		vBanks = new Vector(10, 10);
		vBankIds = new Vector(10, 10);
		iTransNum1 = 0;
		
		openLog();
		bsfOutFrame.show();
		
	}
	
	
	//METHODS
	
	public void closeLog()
	{
		try
		{
			fLogFile.flush();
			fLogFile.close();
		}
		catch(Exception e)
		{
			System.out.print(e);
		}
	}
	
	
	private BankTransactionInterface getBank(int inI)
	{
		return ( (BankTransactionInterface)vBanks.get(inI));
	}
	

	public static void main(String args[])
	{
		BankSwitch 	bs 		= null;
		String 			name 	= null;
		
		try
		{
			System.out.println("Creating BankSwitch");
			bs = new BankSwitch();
			System.out.println("Created, checking for security manager");

			//creating security manager
			if (System.getSecurityManager() == null) 
			{
      	System.setSecurityManager(new RMISecurityManager());
				System.out.println("Security manager created");
      }
			else
			{
				System.out.println("Security manager exists");
			}
      
			//Creating naming reference string
			if(args.length > 0)
			{
				System.out.println("Name specified. using \"/" + args[0] + "\"");
				name = new String("/" + args[0]);
				
			}
			
			else
			{
				System.out.println("No arg[0]. using \"/TransactionInterface\"");
				name = new String("/TransactionInterface");
			}
      
			//Binding the BankSwitch
			System.out.println("Binding BankSwitch");
			Naming.rebind(name, bs);
      System.out.println("BankSwitch bound");
       			
		}
		
		catch(Exception e)
		{
			System.out.println(e.getMessage());
		}
		
	}
	
	
	public boolean openLog() //DONE
	{
		Date h = new Date();
		
		//Create the log file name based on todays date and time.
		cThisDate = new GregorianCalendar();
		cThisDate.setTime(h);
		sTransNum0 = new String(cThisDate.get(GregorianCalendar.YEAR) + "" + cThisDate.get(GregorianCalendar.DAY_OF_YEAR) + "" + cThisDate.get(GregorianCalendar.HOUR_OF_DAY) +  "" + cThisDate.get(GregorianCalendar.MINUTE) + "" + cThisDate.get(GregorianCalendar.SECOND));
		
		//Set the output format for transaction numbers
		df = new DecimalFormat(TRANSACTION_NUMBER_LENGTH);
		
		
		try
		{
			fLogFile = new FileOutputStream(new File(sTransNum0 + ".log"));
			pLogFile = new PrintStream(fLogFile);
			return (true);
		}
		catch(Exception e)
		{
			System.out.print("Error: " + e);
			return (false);
		}
	}
	
	public boolean writeLog(String inLog)
	{
		pLogFile.println(inLog);
		return (true);
	}
	
	
	//INTERFACE

	public boolean join(BankTransactionInterface inBTI)
	{
		BankTransactionInterface 	bTempBank = null;
		boolean 									found 		= false;
		String 										sBankId		= null;
		
		
		try
		{
			sBankId = inBTI.getBankId();
			
			System.out.println(vBanks.size() + " Banks currently joined");
			
			//Check to see if a bank with the same BankId (key) is joined.
			for(int i = 0; i < vBanks.size(); i++)
			{
				bTempBank = getBank(i);
				
				if(sBankId.equals(bTempBank.getBankId()) )
				{
					found = true;
				}
				
			}
			
			//If the bank has not yet joined add them otherwise return false
			if(!found)
			{
				vBanks.add(inBTI);
				vBankIds.add(sBankId);
				System.out.println("New Bank. BankId:" + sBankId);
				return(true);
			}
			
			else
			{
				System.out.println("Bank with Id " + sBankId + " already joined.  Denying access");
				return (false);
			}
			
		}
		
		catch(Exception e)
		{
			
			System.out.println(e.getMessage());
			return(false);
			
		}
		
		
	}
	
	
	//This class is not implemented as it is not in the assignment specification
	public String getRemoteBalance(Transaction inTransaction) throws java.rmi.RemoteException  //NOT IMPLEMENTED
	{
		//write result of transaction
		return (null);
	}


	
	public String getRemoteCash(Transaction inTransaction) throws java.rmi.RemoteException
	{
		BankTransactionInterface btiBank;
		String sLogString;
		String sReturnString;

		
		iTransNum1++;
		
		for(int i = 0; i < vBanks.size(); i++)
		{
			btiBank = (BankTransactionInterface)vBanks.get(i);
			
			if(inTransaction.sAccountNum.startsWith(btiBank.getBankId()))
			{
				sReturnString = btiBank.getRemoteCash(inTransaction);
				System.out.println("Found remote bank returning: " + sReturnString);
				try
				{
			
					sLogString = new String(sTransNum0 + "-" + df.format((long)iTransNum1) + " " + inTransaction.sAccountNum + " " + inTransaction.sType + " " + sReturnString);
					writeLog(sLogString);
					bsfOutFrame.addLog(sLogString);
					return (sReturnString);
					
				}
				catch(Exception e)
				{
					System.out.print(e);
					return (sReturnString);
				}

			}
			
		}
		
		try
		{
			
			sLogString = new String(sTransNum0 + "-" + df.format((long)iTransNum1) + " " + inTransaction.sAccountNum + " " + inTransaction.sType + " " + "NO BANK");
			writeLog(sLogString);
			bsfOutFrame.addLog(sLogString);
			
		}
		catch(Exception f)
		{
			System.out.print(f);
		}
		//write result of transaction
		return("Unable to locate Bank");
		
	}
	
	
	public void remove(String inBankId)
	{
		
		boolean bFound			= false;
		int 		i						= 0;
		String 	sTempBankId = null;
		
		while((i < vBanks.size()) && (bFound == false))
		{
			sTempBankId = (String)vBankIds.get(i);
			if(inBankId.equals(sTempBankId))
			{
				vBanks.remove(i);
				vBankIds.remove(i);
				System.out.println("Removing Bank " + sTempBankId);
			}
		}
	}
	
	
	//VARIABLES
	
	private BankSwitchFrame   bsfOutFrame;
	private FileOutputStream	fLogFile;
	private GregorianCalendar	cThisDate;
	private int								iTransNum1;
	private DecimalFormat			df;
	public  PrintStream				pLogFile;
	private String						sTransNum0;
	private Vector						vBanks;
	private Vector						vBankIds;  //This variable NOT to be modified except in join and remove
	
	//CONSTANTS
	public static final String   TRANSACTION_NUMBER_LENGTH = "000000";
	

}



